/*
 * Copyright (c) OSGi Alliance (2009, 2015). All Rights Reserved.
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.osgi.service.blueprint.reflect;

import org.osgi.annotation.versioning.ConsumerType;

/**
 * Top level Metadata type. All Metadata types extends this base type.
 * 
 * @ThreadSafe
 * @author $Id: 074799a5dd453950d0b14c8612912b7c839858d6 $
 */
@ConsumerType
public interface Metadata {
	// marker interface
}
