/*
 * Copyright (c) OSGi Alliance (2008, 2013). All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.osgi.service.blueprint.reflect;

import org.osgi.annotation.versioning.ConsumerType;

/**
 * Metadata for a value specified to be {@code null} via the &lt;null&gt;
 * element.
 * 
 * @ThreadSafe
 * @author $Id: 43f4d33a2b0fc7126742f6d2f4287b73af4573ad $
 */
@ConsumerType
public interface NullMetadata extends Metadata {

	/**
	 * Singleton instance of {@code NullMetadata}.
	 */
	static final NullMetadata	NULL	= new NullMetadata() {
											// empty body
										};
}
