/*
 * Copyright (c) OSGi Alliance (2010, 2014). All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Rest Service Client Package Version 1.0.
 * 
 * @author $Id: a8f2c7bc6969d859205579b05f4183a5fa348df1 $
 */

@Version("1.0.0")
package org.osgi.service.rest.client;

import org.osgi.annotation.versioning.Version;

